/*
 *  gui_main.cc -- BabyTrans ( Babylon Translator front-end for GTK )
 *
 *  Copyright (C) 1999-2002  Frederic Jolliton -- <babytrans@tuxee.net>
 *  Applet changes Copyright (C) 2000 by Marcus Schiesser
 *  <marcus.schiesser@student.uni-tuebingen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <cctype>
#include <iostream>

using std::isalpha ;

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "gui_main.h"
#include "gui_help.h"
#include "gui_prefs.h"
#include "main.h"
#include "misc.h"
#include "common.h"
#include "str_format.h"
#include "all_pixmaps.h"

namespace {

// FIXME: Use pointer to GdkColor instead ? (to handle NULL as
// default color)
GdkColor			textColor[ guiMain::colorLast ] = {} ;

GtkWidget*			widget = 0 ;

/* The text entry widget to enter word to translate */
GtkWidget*			textWidget = 0 ;

/* The toolbar widget */
GtkWidget*			toolbarWidget = 0 ;

/* The current toolbar style */
GtkToolbarStyle		toolbarStyle = GTK_TOOLBAR_BOTH ;

/* The status bar widget */
GtkWidget*			statusbarWidget = 0 ;

/* The entry widget */
GtkWidget*			entryWidget = 0 ;

bool
allocateGdkColor( GdkColormap* colorMap ,
				  GdkColor* color ,
				  int red , int green , int blue )
{
	color->red = red * 256 ;
	color->green = green * 256 ;
	color->blue = blue * 256 ;
	return bool( gdk_colormap_alloc_color( colorMap , color , TRUE , TRUE ) ) ;
}

void
allocateColors( GdkWindow* window )
{
	GdkColormap*		colorMap = gdk_window_get_colormap( window ) ;
	
	gdk_color_black( colorMap , &textColor[ guiMain::colorNormal ] ) ;
	
	allocateGdkColor( colorMap ,
					  &textColor[ guiMain::colorHighlight ] ,
					  0 , 120 , 255 ) ; // blue

	allocateGdkColor( colorMap ,
					  &textColor[ guiMain::colorError ] ,
					  255 , 0 , 0 ) ; // red
}

void
cbResetPtr( GtkWidget* , gpointer data )
{
	*(GtkWidget**)data = 0 ;
}

void
cbButtonTrans( GtkWidget* widget , gpointer data )
{
	guiMain::translateSelection() ;
}

void
cbButtonSetup( GtkWidget* , gpointer data )
{
	guiSetup::display( widget ) ;
}

#ifdef GNOME_APPLET
void
cbAppletSetup( AppletWidget* caller , gpointer data )
{
	guiSetup::display( widget ) ;
}

void
cbAppletAbout( AppletWidget* caller , gpointer data )
{
	guiHelp::display( widget ) ;
}
#endif

void
cbButtonHelp( GtkWidget* , gpointer data )
{
	guiHelp::display( widget ) ;
}

void
cbButtonIconify( GtkWidget* , gpointer data )
{
	guiMain::iconify() ;
}

void
cbButtonQuit( GtkWidget* , gpointer data )
{
	gtk_main_quit() ;
}

void
cbDelete( GtkWidget* , GdkEvent* , gpointer data )
{
	gtk_main_quit() ;
}

void
cbEntryActivate( GtkWidget* widget , gpointer data )
{
	app::translateWord( gtk_entry_get_text( GTK_ENTRY( entryWidget ) ) ) ;
}

/* FIXME: ugh */
#define GDK_MAIN_MASK (GDK_SHIFT_MASK|GDK_CONTROL_MASK|GDK_MOD1_MASK)

gboolean
cbKeyPress( GtkWidget* widget ,
			GdkEventKey* event ,
			gpointer data )
{
	/* FIXME: stop signal ? */
	int					state = event->state & GDK_MAIN_MASK ;

	switch ( event->keyval ) {
	case GDK_Escape :
		if ( ! app::applet ) {
			gtk_main_quit() ;
		}
		return TRUE ;
	case GDK_space :
		if ( state == GDK_CONTROL_MASK ) {
			guiMain::translateSelection() ;
			return TRUE ;
		}
		break ;
	case GDK_p :
		if ( state == GDK_CONTROL_MASK ) {
			guiSetup::display( 0 ) ;
			return TRUE ;
		}
		break ;
	case GDK_z :
		if ( state == GDK_CONTROL_MASK ) {
			if ( ! app::applet ) {
				guiMain::iconify() ;
			}
			return TRUE ;
		}
		break ;
	case GDK_F1 :
		if ( state == 0 ) {
			guiHelp::display( widget ) ;
			return TRUE ;
		}
		break ;
	}

	return FALSE ;
}

void
getSelection( GtkWidget* widget ,
			  GtkSelectionData* selection_data , guint ,
			  gpointer data )
{
	if ( selection_data->length < 0 ) {
#if 0
		gtk_editable_delete_text( GTK_EDITABLE( guiMain::textWidget ) ,
									0 , -1 ) ;
		gtk_text_insert( GTK_TEXT( guiMain::textWidget ) ,
							0 , color[ colorError ] , 0 ,
							_( "Invalid selection" ) , -1 ) ;
#endif
		return ;
	}
	std::string				selection( selection_data->data ,
									selection_data->data
									+ selection_data->length ) ;
	size_t				i ;
	for ( i = 0 ; i < selection.size() ; ++ i ) {
		if ( isalpha( selection[ i ] ) )
			break ;
	}
	selection.erase( 0 , i ) ;
	for ( i = 0 ; i < selection.size() ; ++ i ) {
		if ( ! isalpha( selection[ i ] )
			&& selection[ i ] != '\'' )
			break ;
	}
	selection.erase( i ) ;
	if ( selection.empty() ) {
#if 0
		gtk_editable_delete_text( GTK_EDITABLE( guiMain::textWidget ) ,
									0 , -1 ) ;
		gtk_text_insert( GTK_TEXT( guiMain::textWidget ) ,
							0 , color[ colorError ] , 0 ,
							_( "No word selected" ) , -1 ) ;
#endif
		return ;
	}
	app::translateWord( selection ) ;
}

} // anonymous namespace

//--[ public ]-----------------------------------------------------------------

// FIXME: Alternative solution to avoid exporting this array to other module ?
/* List of icons for the toobar */
guiMain::ToolbarItem	guiMain::toolbarDescription[] = {
	{
		Refresh_xpm ,
		N_( "Translate" ) ,
		N_( "Translate the current selection" ) ,
		N_( "Ctrl-Space" ) ,
		cbButtonTrans ,
		true
	} , {
		Preferences_xpm ,
		N_( "Pref." ) ,
		N_( "Display the preference dialog" ) ,
		N_( "Ctrl-P" ) ,
		cbButtonSetup ,
		true
	} ,	{
		Help_xpm ,
		N_( "Help" ) ,
		N_( "Display the help" ) ,
		N_( "F1" ) ,
		cbButtonHelp ,
		true
	} , {
		Stop_xpm ,
		N_( "Iconize" ) ,
		N_( "Iconize the window" ) ,
		N_( "Ctrl-Z" ) ,
		cbButtonIconify ,
		false
	} ,	{
		Quit_xpm ,
		N_( "Quit" ) ,
		N_( "Quit babytrans" ) ,
		N_( "ESC" ) ,
		cbButtonQuit ,
		false
	} , {
		0 ,
		0 ,
		0 ,
		0 ,
		0 ,
		false
	}
} ;

/*
 * Needed for Help and Setup dialog box to set it as transient.
 */
GtkWidget*
guiMain::getWidget()
{
	return widget ;
}

void
guiMain::translateSelection()
{
	static GdkAtom		selection_atom = GDK_NONE ;

	if ( selection_atom == GDK_NONE ) {
		selection_atom = gdk_atom_intern( "STRING" , FALSE ) ;
		if ( selection_atom == GDK_NONE )
			; /* FIXME: Do error handling */
	}

	if ( widget == 0 ) { return ; }

	if ( gtk_selection_convert( widget ,
								GDK_SELECTION_PRIMARY ,
								selection_atom ,
								GDK_CURRENT_TIME ) == FALSE )
		; /* FIXME: Do error handling */
}

std::string
guiMain::getTextEntry()
{
	if ( entryWidget == 0 ) {
		return std::string() ;
	}
	return std::string( gtk_entry_get_text( GTK_ENTRY( entryWidget ) ) ) ;
}

void
guiMain::setTextEntry( const std::string& text )
{
	if ( entryWidget != 0 ) {
		gtk_entry_set_text( GTK_ENTRY( entryWidget ) , text.c_str() ) ;
	}
}

void
guiMain::clearText()
{
	// Clear the previous translation text
	gtk_editable_delete_text( GTK_EDITABLE( textWidget ) , 0 , -1 ) ;
}

void
guiMain::insertText( const std::string& text , guiMain::Color color )
{
	gtk_text_insert( GTK_TEXT( textWidget ) ,
					 0 , &textColor[ color ] , 0 ,
					 text.c_str() , text.size() ) ;
}

void
guiMain::freezeText()
{
	gtk_text_freeze( GTK_TEXT( textWidget ) ) ;
}

void
guiMain::thawText()
{
	gtk_text_thaw( GTK_TEXT( textWidget ) ) ;
}

void
guiMain::grabFocus()
{
	gtk_widget_grab_focus( textWidget ) ;
}

void
guiMain::raiseAndFocus()
{
	fred_gtk_window_raise_and_focus( widget ) ;
}

void
guiMain::iconify()
{
	fred_gtk_window_iconify( GTK_WINDOW( widget ) ) ;
}

void
guiMain::setToolbarStyle( GtkToolbarStyle s )
{
	toolbarStyle = s ;
	if ( toolbarWidget != 0 )
		gtk_toolbar_set_style( GTK_TOOLBAR( toolbarWidget ) , s ) ;
}

GtkToolbarStyle
guiMain::getToolbarStyle()
{
	return toolbarStyle ;
}

/*
 * Update status bar
 */
void
guiMain::update()
{
	GtkStatusbar*		statusbar = (GtkStatusbar*)statusbarWidget ;
	if( statusbar != 0 ) {
		std::string				str = strFormat( "[%1] - %2" )
			.with( app::languages[ app::currentDictionaryIndex ].longName )
			.with( app::getAuto() ? _( "Auto mode" ) : _( "Normal mode" ) ) ;
		guint				id =
			gtk_statusbar_get_context_id( statusbar , "keskecekeca" ) ;
		gtk_statusbar_pop( statusbar , id ) ;
		gtk_statusbar_push( statusbar , id , str.c_str() ) ;
		gtk_window_set_title( GTK_WINDOW( widget ) ,
								strFormat( "%1 - '%2'" )
								.with( PACKAGE " " VERSION )
								.with( app::languages[ app::currentDictionaryIndex ]
										.longName.c_str() ) ) ;
	}
}

/* Create the window
 *
 */
void
guiMain::display()
{
	if ( widget != 0 ) { // already created ?
		gtk_widget_show( widget ) ;
		return ;
	}
	GtkToolbar *tb;
	GtkWidget *pix, *top_hbox, *bar, *hbox, *vbox;

#ifdef GNOME_APPLET
	if ( ! ( widget = applet_widget_new( PACKAGE ) ) ) {
		g_error( "Can't create applet!\n" ) ;
	}
	// make applet menu
	applet_widget_register_stock_callback( APPLET_WIDGET( widget ) ,
											"setup" ,
											GNOME_STOCK_MENU_PROP ,
											_( "Setup..." ) ,
											&guiMain::cbAppletSetup ,
											NULL ) ;
	// add the about box
	applet_widget_register_stock_callback( APPLET_WIDGET( widget ) ,
											"about" ,
											GNOME_STOCK_MENU_ABOUT ,
											_( "About..." ) ,
											&guiMain::cbAppletAbout ,
											NULL ) ;
#else
	widget = gtk_window_new( GTK_WINDOW_TOPLEVEL ) ;
	gtk_widget_realize( widget ) ;

	allocateColors( widget->window ) ;

	/* toolbar */
	toolbarWidget = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL ,
									 toolbarStyle ) ;
	gtk_toolbar_set_button_relief( GTK_TOOLBAR( toolbarWidget ) ,
								   GTK_RELIEF_NONE ) ;
	tb = GTK_TOOLBAR( toolbarWidget ) ;
	for ( size_t i = 0 ; guiMain::toolbarDescription[ i ].xpmData != 0 ; ++ i ) {
		pix = 0 ;
		if ( guiMain::toolbarDescription[ i ].xpmData != 0 )
			pix = createPixmap( guiMain::toolbarDescription[ i ].xpmData ,
									GTK_WIDGET( widget )->window ) ;
		gtk_toolbar_append_item( tb ,
									_( guiMain::toolbarDescription[ i ].name ) ,
									strFormat( "%1 (%2)" )
									.with( _( guiMain::toolbarDescription[ i ].tips ) )
									.with( _( guiMain::toolbarDescription[ i ].key ) ) ,
									0 ,
									pix ,
									GTK_SIGNAL_FUNC( guiMain::toolbarDescription[ i ].cb ) ,
									0 ) ;
	}
	gtk_widget_show( toolbarWidget ) ;

		/* entry box */
	entryWidget = gtk_entry_new() ;
	gtk_widget_set_usize( entryWidget ,
						  100 ,
						  entryWidget->style->font->ascent +
						  entryWidget->style->font->descent +
						  ( entryWidget->style->klass->ythickness + 2 ) * 2 ) ;
	connect( entryWidget , "activate" ,
			 cbEntryActivate , 0 ) ;
	gtk_widget_show( entryWidget ) ;

	/* layouter for toolbar & entry */
	top_hbox = gtk_hbox_new( FALSE , 2 * guiSpace ) ;
	gtk_box_pack_start( GTK_BOX( top_hbox ) , toolbarWidget ,
						FALSE , FALSE , 0 ) ;
	gtk_box_pack_start( GTK_BOX( top_hbox ) , entryWidget ,
						TRUE , TRUE , 0 ) ;
	gtk_widget_show( top_hbox ) ;
#endif

	/* text */
	textWidget = gtk_text_new( 0 , 0 ) ;
#ifdef GNOME_APPLET
	// add an option to set the size ? Or, better, ask panel for its size ?
	gtk_widget_set_usize( textWidget , 128 , 32 ) ;
#else
	gtk_widget_set_usize( textWidget , 200 , 100 ) ;
#endif

	gtk_text_set_word_wrap( GTK_TEXT( textWidget ) ,
							TRUE ) ;

	bar = gtk_vscrollbar_new( GTK_TEXT( textWidget )->vadj ) ;
	hbox = gtk_hbox_new( FALSE , guiSpace / 2 ) ;
	gtk_container_add( GTK_CONTAINER( hbox ) , textWidget ) ;
	gtk_box_pack_start( GTK_BOX( hbox ) , bar , FALSE , FALSE , 0 ) ;
	gtk_widget_show_all( hbox ) ;

#ifdef GNOME_APPLET
	applet_widget_add( APPLET_WIDGET( widget ) , hbox ) ;
#else
	/* status bar */
	statusbarWidget = gtk_statusbar_new() ;
	gtk_widget_show( statusbarWidget ) ;

		/* main vbox */
	vbox = gtk_vbox_new( FALSE , guiSpace ) ;
	gtk_box_pack_start( GTK_BOX( vbox ) , top_hbox ,
						FALSE , FALSE , 0 ) ;
	gtk_container_add( GTK_CONTAINER( vbox ) , hbox ) ;
	gtk_box_pack_start( GTK_BOX( vbox ) , statusbarWidget ,
						FALSE , FALSE , 0 ) ;
	gtk_widget_show( vbox ) ;

	gtk_container_add( GTK_CONTAINER( widget ) , vbox ) ;
	gtk_container_set_border_width( GTK_CONTAINER( widget ) ,
									guiSpace ) ;
#endif

	connect( widget , "selection-received" , getSelection , 0 ) ;
	connect( widget , "delete-event" , cbDelete , 0 ) ;
	connect( widget , "key-press-event" , cbKeyPress , 0 ) ;
	connect( widget , "destroy" , cbResetPtr , &widget ) ;

	gtk_widget_show( widget ) ;
}

// vim: set ts=4 sw=4:
