/*
 *  misc.cc -- BabyTrans ( Babylon Translator front-end for GTK )
 *
 *  Copyright (C) 1999-2002  Frederic Jolliton -- <babytrans@tuxee.net>
 *  Applet changes Copyright (C) 2000 by Marcus Schiesser
 *  <marcus.schiesser@student.uni-tuebingen.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

/* Create a GTK pixmap from XPM data */
GtkWidget*
createPixmap( const gchar** data , GdkWindow* window )
{
	GdkBitmap*			mask ;
	GdkPixmap*			pixmap ;
	GtkWidget*			widget ;

	pixmap = gdk_pixmap_create_from_xpm_d( window , &mask , 0 ,
											const_cast< gchar** >( data ) ) ;
	widget = gtk_pixmap_new( pixmap , mask ) ;

	gdk_pixmap_unref( pixmap ) ;
	gdk_pixmap_unref( mask ) ;

	return widget ;
}

/* Iconify the specified GTK window (must be toplevel) */
void
fred_gtk_window_iconify( GtkWindow* window )
{
	GtkWidget*			widget = GTK_WIDGET( window ) ;
	if ( ! GTK_WIDGET_TOPLEVEL( widget ) )
		return ;
	XIconifyWindow( GDK_WINDOW_XDISPLAY( widget->window ) ,
					GDK_WINDOW_XWINDOW( widget->window ) ,
					DefaultScreen( GDK_DISPLAY() ) ) ;
}

/* Raise a GTK window and give it the focus */
void
fred_gtk_window_raise_and_focus( GtkWidget* widget )
{
	g_assert( GTK_WIDGET_REALIZED( widget ) ) ;
	gdk_window_show( widget->window ) ;
	gtk_widget_grab_focus( widget ) ;
}
